@@ -74,7 +74,9 @@ struct _GnomeCmdDataPrivate
 	gboolean            device_only_icon;
 	gint                dir_cache_size;
 	gboolean            use_ls_colors;
-	GnomeCmdConFtp      *quick_connect_server;
+	gchar               *quick_connect_host;
+	gchar               *quick_connect_user;
+	gint                quick_connect_port;
 	gboolean            honor_expect_uris;
 	gboolean            skip_mounting;
 	gboolean            toolbar_visibility;
@@ -88,6 +90,11 @@ struct _GnomeCmdDataPrivate
 	gchar               *last_pattern;
 	GList               *auto_load_plugins;
 	guint               gui_update_rate;
+	gint                sort_column[2];
+	gboolean            sort_direction[2];
+	gint                main_win_pos[2];
+	gchar               *backup_pattern;
+	GList               *backup_pattern_list;
 	
 	gchar *viewer;
 	gchar *editor;
@@ -958,8 +965,24 @@ gnome_cmd_data_save                      (void)
 	gnome_cmd_data_set_bool   ("/programs/toolbar_visibility",
 				data->priv->toolbar_visibility);
 	gnome_cmd_data_set_bool   ("/programs/buttonbar_visibility",
-				data->priv->buttonbar_visibility);
+							   data->priv->buttonbar_visibility);
+
+	gnome_cmd_data_set_int (
+		"/options/main_win_pos_x",
+		data->priv->main_win_pos[0]);
+	gnome_cmd_data_set_int (
+		"/options/main_win_pos_y",
+		data->priv->main_win_pos[1]);
 	
+	gnome_cmd_data_set_int (
+		"/options/sort_column_left", data->priv->sort_column[0]);
+	gnome_cmd_data_set_bool (
+		"/options/sort_direction_left", data->priv->sort_direction[0]);
+	gnome_cmd_data_set_int (
+		"/options/sort_column_right", data->priv->sort_column[1]);
+	gnome_cmd_data_set_bool (
+		"/options/sort_direction_right", data->priv->sort_direction[1]);
+
 	
 	gnome_cmd_data_set_string ("/programs/viewer",
 				data->priv->viewer);
@@ -978,16 +1001,32 @@ gnome_cmd_data_save                      (void)
 				data->priv->use_ls_colors);
 
 	gnome_cmd_data_set_string ("/quick-connect/host",
-				gnome_cmd_con_ftp_get_host_name (data->priv->quick_connect_server));
+							   data->priv->quick_connect_host);
 	gnome_cmd_data_set_int    ("/quick-connect/port",		
-				gnome_cmd_con_ftp_get_host_port (data->priv->quick_connect_server));
+							   data->priv->quick_connect_port);
 	gnome_cmd_data_set_string ("/quick-connect/user",
-				gnome_cmd_con_ftp_get_user_name (data->priv->quick_connect_server));
+							   data->priv->quick_connect_user);
+
+	gnome_config_set_int (
+		"/gnome-commander-size/main_win/width",
+		data->priv->main_win_width);
+	gnome_config_set_int (
+		"/gnome-commander-size/main_win/height",
+		data->priv->main_win_height);
+
+	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ ) {
+		gchar *tmp = g_strdup_printf (
+			"/gnome-commander-size/column-widths/fs_col_width%d", i);
+		gnome_config_set_int (tmp, data->priv->fs_col_width[i]);
+		g_free (tmp);
+	}
 	
 	gnome_cmd_data_set_string ("/options/start_dir_left", data->priv->start_dirs[0]);
 	gnome_cmd_data_set_string ("/options/start_dir_right", data->priv->start_dirs[1]);
 	gnome_cmd_data_set_string ("/defaults/last_pattern",
 				data->priv->last_pattern);
+	gnome_cmd_data_set_string ("/defaults/backup_pattern",
+				data->priv->backup_pattern);
 	
 	write_cmdline_history ();
 	//write_dir_history ();
@@ -1027,11 +1066,11 @@ gnome_cmd_data_load                      (void)
 
 	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].respect_theme = FALSE;
 	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].norm_fg = gdk_color_new (0,0,0);
-	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].norm_bg = gdk_color_new (0xffff,0xffff,0xffff);
+	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].norm_bg = gdk_color_new (0xdddd,0xdddd,0xdddd);
 	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].sel_fg = gdk_color_new (0xffff,0,0);
-	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].sel_bg = gdk_color_new (0xffff,0xffff,0xffff);
+	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].sel_bg = gdk_color_new (0xdddd,0xdddd,0xdddd);
 	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].curs_fg = gdk_color_new (0xffff,0xffff,0xffff);
-	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].curs_bg = gdk_color_new (0,0,0x6666);
+	data->priv->color_themes[GNOME_CMD_COLOR_MODERN].curs_bg = gdk_color_new (0,0,0x4444);
 	
 	data->priv->color_themes[GNOME_CMD_COLOR_CLASSIC].respect_theme = FALSE;
 	data->priv->color_themes[GNOME_CMD_COLOR_CLASSIC].norm_fg = gdk_color_new (0xffff,0xffff,0xffff);
@@ -1177,6 +1216,10 @@ gnome_cmd_data_load                      (void)
 		"/options/cmdline_visibility", TRUE);
 	data->priv->gui_update_rate = gnome_cmd_data_get_int (
 		"/options/gui_update_rate", DEFAULT_GUI_UPDATE_RATE);
+	data->priv->main_win_pos[0] = gnome_cmd_data_get_int (
+		"/options/main_win_pos_x", -1);
+	data->priv->main_win_pos[1] = gnome_cmd_data_get_int (
+		"/options/main_win_pos_y", -1);
 
 	if (data->priv->gui_update_rate < MIN_GUI_UPDATE_RATE)
 		data->priv->gui_update_rate = MIN_GUI_UPDATE_RATE;
@@ -1192,6 +1235,15 @@ gnome_cmd_data_load                      (void)
 	data->priv->buttonbar_visibility = gnome_cmd_data_get_bool (
 		"/programs/buttonbar_visibility", TRUE);
 
+	data->priv->sort_column[0] = gnome_cmd_data_get_int (
+		"/options/sort_column_left", 1);
+	data->priv->sort_direction[0] = gnome_cmd_data_get_bool (
+		"/options/sort_direction_left", FALSE);
+	data->priv->sort_column[1] = gnome_cmd_data_get_int (
+		"/options/sort_column_right", 1);
+	data->priv->sort_direction[1] = gnome_cmd_data_get_bool (
+		"/options/sort_direction_right", FALSE);
+
 	data->priv->viewer = gnome_cmd_data_get_string ("/programs/viewer",
 									 "emacs %s");
 	data->priv->editor = gnome_cmd_data_get_string ("/programs/editor",
@@ -1215,13 +1267,17 @@ gnome_cmd_data_load                      (void)
 		"/options/start_dir_right", g_get_home_dir ());
 	data->priv->last_pattern = gnome_cmd_data_get_string (
 		"/defaults/last_pattern", "");
+	data->priv->backup_pattern = gnome_cmd_data_get_string (
+		"/defaults/backup_pattern", "*~;*.bak");
+	data->priv->backup_pattern_list = patlist_new (data->priv->backup_pattern);
 
-	data->priv->quick_connect_server = gnome_cmd_con_ftp_new (
-		"tmp",
-		gnome_cmd_data_get_string ("/quick-connect/host", "ftp.gnome.org"),
-		gnome_cmd_data_get_int ("/quick-connect/port", 21),
-		gnome_cmd_data_get_string ("/quick-connect/user", "anonymous"),
-		data->priv->ftp_anonymous_password);
+
+	data->priv->quick_connect_host = gnome_cmd_data_get_string (
+		"/quick-connect/host", "ftp.gnome.org");
+	data->priv->quick_connect_port = gnome_cmd_data_get_int (
+		"/quick-connect/port", 21);
+	data->priv->quick_connect_user = gnome_cmd_data_get_string (
+		"/quick-connect/user", "anonymous");
 	
 	
 	load_cmdline_history ();
@@ -1377,6 +1433,13 @@ gnome_cmd_data_get_current_color_theme (void)
 }
 
 
+GnomeCmdColorTheme*
+gnome_cmd_data_get_custom_color_theme (void)
+{
+	return &data->priv->color_themes[GNOME_CMD_COLOR_CUSTOM];
+}
+
+
 gint
 gnome_cmd_data_get_list_row_height                (void)
 {
@@ -1406,7 +1469,7 @@ gnome_cmd_data_get_ext_disp_mode        (void)
 
 
 void
-gnome_cmd_data_set_position             (gint width, gint height)
+gnome_cmd_data_set_main_win_size        (gint width, gint height)
 {
 	data->priv->main_win_width = width;
 	data->priv->main_win_height = height;
@@ -1414,34 +1477,13 @@ gnome_cmd_data_set_position             (gint width, gint height)
 
 
 void
-gnome_cmd_data_get_position             (gint *width, gint *height)
+gnome_cmd_data_get_main_win_size        (gint *width, gint *height)
 {
 	*width = data->priv->main_win_width;
 	*height = data->priv->main_win_height;
 }
 
 
-void
-gnome_cmd_data_save_position            (void)
-{
-	gint i;
-	
-	gnome_config_set_int (
-		"/gnome-commander-size/main_win/width",
-		data->priv->main_win_width);
-	gnome_config_set_int (
-		"/gnome-commander-size/main_win/height",
-		data->priv->main_win_height);
-
-	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ ) {
-		gchar *tmp = g_strdup_printf (
-			"/gnome-commander-size/column-widths/fs_col_width%d", i);
-		gnome_config_set_int (tmp, data->priv->fs_col_width[i]);
-		g_free (tmp);
-	}
-}
-
-
 void
 gnome_cmd_data_set_viewer               (const gchar *command)
 {
@@ -1804,10 +1846,47 @@ gnome_cmd_data_get_search_defaults (void)
 }
 
 
-GnomeCmdConFtp *
-gnome_cmd_data_get_quick_connect_server (void)
+const gchar *
+gnome_cmd_data_get_quick_connect_host  (void)
 {
-	return data->priv->quick_connect_server;
+	return data->priv->quick_connect_host;
+}
+
+
+const gchar *
+gnome_cmd_data_get_quick_connect_user  (void)
+{
+	return data->priv->quick_connect_user;
+}
+
+
+gint
+gnome_cmd_data_get_quick_connect_port  (void)
+{
+	return data->priv->quick_connect_port;
+}
+
+
+void
+gnome_cmd_data_set_quick_connect_host  (const gchar *value)
+{
+	g_free (data->priv->quick_connect_host);
+	data->priv->quick_connect_host = g_strdup (value);
+}
+
+
+void
+gnome_cmd_data_set_quick_connect_user  (const gchar *value)
+{
+	g_free (data->priv->quick_connect_user);
+	data->priv->quick_connect_user = g_strdup (value);
+}
+
+
+void
+gnome_cmd_data_set_quick_connect_port  (gint value)
+{
+	data->priv->quick_connect_port = value;
 }
 
 
@@ -1974,3 +2053,76 @@ gnome_cmd_data_get_gui_update_rate (void)
 }
 
 
+void
+gnome_cmd_data_get_sort_params (GnomeCmdFileList *fl, gint *col, gboolean *direction)
+{
+	if (!gnome_cmd_main_win_get_left_fs (main_win) ||
+		gnome_cmd_main_win_get_left_fs (main_win)->list == fl) {
+		*col = data->priv->sort_column[0];
+		*direction = data->priv->sort_direction[0];
+	}
+	else if (!gnome_cmd_main_win_get_right_fs (main_win) ||
+			 gnome_cmd_main_win_get_right_fs (main_win)->list == fl) {
+		*col = data->priv->sort_column[1];
+		*direction = data->priv->sort_direction[1];
+	}
+}
+
+
+void
+gnome_cmd_data_set_sort_params (GnomeCmdFileList *fl, gint col, gboolean direction)
+{
+	if (gnome_cmd_main_win_get_left_fs (main_win)->list == fl) {
+		data->priv->sort_column[0] = col;
+		data->priv->sort_direction[0] = direction;
+	}
+	else if (gnome_cmd_main_win_get_right_fs (main_win)->list == fl) {
+		data->priv->sort_column[1] = col;
+		data->priv->sort_direction[1] = direction;
+	}
+}
+
+
+void
+gnome_cmd_data_set_main_win_pos (gint x, gint y)
+{
+	data->priv->main_win_pos[0] = x;
+	data->priv->main_win_pos[1] = y;
+}
+
+
+void
+gnome_cmd_data_get_main_win_pos (gint *x, gint *y)
+{
+	*x = data->priv->main_win_pos[0];
+	*y = data->priv->main_win_pos[1];
+}
+
+
+void
+gnome_cmd_data_set_backup_pattern (const gchar *value)
+{
+	if (data->priv->backup_pattern)
+		g_free (data->priv->backup_pattern);
+	
+	data->priv->backup_pattern = g_strdup (value);
+
+	if (data->priv->backup_pattern_list)
+		patlist_free (data->priv->backup_pattern_list);
+
+	data->priv->backup_pattern_list = patlist_new (data->priv->backup_pattern);
+}
+
+
+const gchar *
+gnome_cmd_data_get_backup_pattern (void)
+{
+	return data->priv->backup_pattern;
+}
+
+
+GList *
+gnome_cmd_data_get_backup_pattern_list (void)
+{
+	return data->priv->backup_pattern_list;
+}
