@@ -47,7 +47,7 @@
 /* The time (in ms) it takes from that the right mouse button is clicked on a file until 
  * the popup menu appears when the right btn is used to select files.
  */
-#define POPUP_TIMEOUT 1000
+#define POPUP_TIMEOUT 750
 
 
 enum {
@@ -104,6 +104,30 @@ GtkJustification list_column_justification[FILE_LIST_NUM_COLUMNS] = {
 };
 
 
+static gint sort_by_name (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_extension (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_dir (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_size (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_perm (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_date (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_owner (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_group (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+
+
+static GnomeVFSListCompareFunc sort_funcs[] =
+{
+	NULL,
+	(GnomeVFSListCompareFunc)sort_by_name,
+	(GnomeVFSListCompareFunc)sort_by_extension,
+	(GnomeVFSListCompareFunc)sort_by_dir,
+	(GnomeVFSListCompareFunc)sort_by_size,
+	(GnomeVFSListCompareFunc)sort_by_date,
+	(GnomeVFSListCompareFunc)sort_by_perm,
+	(GnomeVFSListCompareFunc)sort_by_owner,
+	(GnomeVFSListCompareFunc)sort_by_group
+};
+
+
 struct _GnomeCmdFileListPrivate {
 	GtkWidget *column_pixmaps[FILE_LIST_NUM_COLUMNS];
 	GtkWidget *column_labels[FILE_LIST_NUM_COLUMNS];
@@ -122,6 +146,7 @@ struct _GnomeCmdFileListPrivate {
 	guint right_mb_timeout_id;
 	GtkWidget *selpat_dialog;
 	GtkWidget *quicksearch_popup;
+	gchar *focus_later;
 };
 
 
@@ -984,37 +1009,9 @@ on_column_clicked                        (GtkCList *list,
 {
 	g_return_if_fail (GTK_IS_CLIST (list));
 	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
-	
-	switch (col)
-	{
-		case 1:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_name;
-			break;
-		case 2:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_extension;
-			break;
-		case 3:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_dir;
-			break;
-		case 4:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_size;
-			break;
-		case 5:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_date;
-			break;
-		case 6:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_perm;
-			break;
-		case 7:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_owner;
-			break;
-		case 8:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_group;
-			break;
-		default:
-			return;
-	}
 
+	fl->priv->sort_func = sort_funcs[col];
+	
 	if (fl->priv->current_col == col)
 		fl->priv->sort_raising[col] = !fl->priv->sort_raising[col];
 
@@ -1022,6 +1019,8 @@ on_column_clicked                        (GtkCList *list,
 	update_column_sort_arrows (fl);
 	
 	gnome_cmd_file_list_sort (fl);
+
+	gnome_cmd_data_set_sort_params (fl, col, fl->priv->sort_raising[col]);
 }
 
 
@@ -1317,17 +1316,25 @@ static void
 init (GnomeCmdFileList *fl)
 {
 	gint i;
+	gboolean b;
 	
 	fl->priv = g_new (GnomeCmdFileListPrivate, 1);
 	fl->priv->shown_files = gnome_cmd_file_collection_new ();
 	fl->priv->selected_files = NULL;
 	fl->priv->shift_down = FALSE;
-	fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_name;
-	fl->priv->current_col = 1;
 	fl->priv->selpat_dialog = NULL;
 	fl->priv->right_mb_down_file = NULL;
 	fl->priv->right_mb_timeout_id = 0;
 	fl->priv->quicksearch_popup = NULL;
+	fl->priv->focus_later = NULL;
+
+	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
+		fl->priv->sort_raising[i] = FALSE;
+	
+	gnome_cmd_data_get_sort_params (fl, &i, &b);
+	fl->priv->current_col = i;
+	fl->priv->sort_raising[i] = b;
+	fl->priv->sort_func = sort_funcs[i];
 	
 	list_column_titles[FILE_LIST_COLUMN_ICON]  = "";
 	list_column_titles[FILE_LIST_COLUMN_NAME]  = _("name");
@@ -1339,9 +1346,6 @@ init (GnomeCmdFileList *fl)
 	list_column_titles[FILE_LIST_COLUMN_OWNER] = _("uid");
 	list_column_titles[FILE_LIST_COLUMN_GROUP] = _("gid");
 	
-	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
-		fl->priv->sort_raising[i] = FALSE;
-
 	init_dnd (fl);
 	
 	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
@@ -1555,6 +1559,11 @@ add_file_to_clist (GnomeCmdFileList *fl, GnomeCmdFile *finfo, gint in_row)
 							  gnome_cmd_file_get_type_pixmap (finfo),
 							  gnome_cmd_file_get_type_mask (finfo));
 	}
+
+	/* If we have been waiting for this file to show up, focus it */
+	if (fl->priv->focus_later &&
+		strcmp (gnome_cmd_file_get_name (finfo), fl->priv->focus_later) == 0)
+		focus_file_at_row (fl, row);
 }
 
 
@@ -2001,6 +2010,12 @@ gnome_cmd_file_list_focus_file (GnomeCmdFileList *fl,
 		}
 		tmp = tmp->next;
 	}
+
+	/* The file was not found, remember the filename in case the file gets
+	   added to the list in the future (after a FAM event etc). */
+	if (fl->priv->focus_later)
+		g_free (fl->priv->focus_later);
+	fl->priv->focus_later = g_strdup (focus_file);
 }
 
 
@@ -2163,8 +2178,9 @@ gnome_cmd_file_list_sort (GnomeCmdFileList *fl)
 	}
 	g_list_free (list);
 
-	/* refocus the previously selected file */
-	if (selfile) {
+	/* refocus the previously selected file if this
+	   file-list has the focus */
+	if (selfile && GTK_WIDGET_HAS_FOCUS (fl)) {
 		selrow = get_row_from_file (fl, selfile);
 		gnome_cmd_file_list_select_row (fl, selrow);
 		gtk_clist_moveto (GTK_CLIST (fl), selrow, -1, 1, 0);
@@ -2424,11 +2440,12 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 				break;
 		}
 	}
-	else if (state_is_ctrl_alt (event->state)) {
+	else if (state_is_ctrl_alt (event->state) || state_is_ctrl_alt_shift (event->state)) {
 		if ((event->keyval >= GDK_a && event->keyval <= GDK_z)
-			|| (event->keyval >= GDK_A && event->keyval <= GDK_Z))
+			|| (event->keyval >= GDK_A && event->keyval <= GDK_Z)
+			|| event->keyval == GDK_period)
 			gnome_cmd_file_list_show_quicksearch (fl, (gchar)event->keyval);
-	}	
+	}
 	else if (state_is_shift (event->state)) {
 		switch (event->keyval)
 		{
@@ -2439,6 +2456,10 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 			case GDK_F6:
 				gnome_cmd_file_list_show_rename_dialog (fl);
 				return TRUE;
+
+			case GDK_F10:
+				show_file_popup_with_warp (fl);
+				return TRUE;
 				
 			case GDK_Left:
 			case GDK_Right:
@@ -2538,10 +2559,12 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 				return mime_exec_file (gnome_cmd_file_list_get_focused_file (fl));
 				
 			case GDK_KP_Add:
+			case GDK_plus:
 				show_selpat_dialog (fl, TRUE);
 				return TRUE;
 
 			case GDK_KP_Subtract:
+			case GDK_minus:
 				show_selpat_dialog (fl, FALSE);
 				return TRUE;
 
