@@ -51,7 +51,6 @@ static GtkTargetEntry drop_types [] = {
 static GtkVBoxClass *parent_class = NULL;
 
 struct _GnomeCmdFileSelectorPrivate {
-	GtkWidget *dir_history_popup;
 	GtkWidget *filter_box;
 	
 	gboolean active;
@@ -85,6 +84,23 @@ static guint file_selector_signals[LAST_SIGNAL] = { 0 };
  * Utility functions
  *******************************/
 
+static void
+set_same_directory (GnomeCmdFileSelector *fs)
+{
+	GnomeCmdFileSelector *other;
+	GnomeCmdCon *con;
+	GnomeCmdDir *dir;
+
+	other = gnome_cmd_main_win_get_inactive_fs (main_win);
+	con = gnome_cmd_file_selector_get_connection (other);
+	dir = gnome_cmd_file_selector_get_directory (other);
+	if (fs->priv->con != con)
+		gnome_cmd_file_selector_set_connection (fs, con, dir);
+	else
+		gnome_cmd_file_selector_set_directory (fs, dir);
+}
+
+
 static void
 show_list_popup (GnomeCmdFileSelector *fs)
 {
@@ -260,8 +276,8 @@ file_is_wanted (GnomeCmdFileSelector *fs, GnomeVFSFileInfo *info)
 		if (info->name[0] == '.' 
 			&& gnome_cmd_data_get_hidden_filter ())
 			return FALSE;
-		if (info->name[strlen(info->name)-1] == '~'
-			&& gnome_cmd_data_get_backup_filter ())
+		if (gnome_cmd_data_get_backup_filter () &&
+			patlist_matches (gnome_cmd_data_get_backup_pattern_list (), info->name))
 			return FALSE;
 
 		return TRUE;
@@ -726,7 +742,7 @@ static void
 goto_directory (GnomeCmdFileSelector *fs,
 				const gchar *in_dir)
 {
-	GnomeCmdDir *cur_dir, *new_dir;
+	GnomeCmdDir *cur_dir, *new_dir = NULL;
 	const gchar *focus_dir = NULL;
 	gchar *dir;
 
@@ -752,14 +768,21 @@ goto_directory (GnomeCmdFileSelector *fs,
 	}
 	else {
 		/* check if it's an absolute address or not */
-		if (dir[0] == '/')
+		if (dir[0] == '/') {
 			new_dir = gnome_cmd_dir_new (
 				fs->priv->con, gnome_cmd_con_create_path (fs->priv->con, dir));
+		}
+		else if (strncmp (dir, "\\\\", 2) == 0) {
+			GnomeCmdPath *path = gnome_cmd_con_create_path (get_smb_con (), dir);
+			if (path)
+				new_dir = gnome_cmd_dir_new (get_smb_con (), path);
+		}
 		else
 			new_dir = gnome_cmd_dir_get_child (cur_dir, dir);			
 	}
 
-	gnome_cmd_file_selector_set_directory (fs, new_dir);
+	if (new_dir)
+		gnome_cmd_file_selector_set_directory (fs, new_dir);
 	
 	/* focus the current dir when going back to the parent dir */
 	if (focus_dir)
@@ -897,6 +920,7 @@ on_con_combo_item_selected              (GnomeCmdCombo *con_combo,
 	g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
 	g_return_if_fail (GNOME_CMD_IS_CON (con));
 
+	gnome_cmd_main_win_switch_fs (main_win, fs);
 	gnome_cmd_file_selector_set_connection (fs, con, NULL);
 }
 
@@ -908,7 +932,7 @@ on_combo_popwin_hidden              (GnomeCmdCombo *combo,
 	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
 	g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
 
-	gtk_widget_grab_focus (fs->list_widget);
+	gnome_cmd_main_win_refocus (main_win);
 }
 
 
@@ -921,6 +945,7 @@ on_con_btn_clicked                      (GtkButton *button,
 	g_return_if_fail (GNOME_CMD_IS_CON (con));
 	g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
 
+	gnome_cmd_main_win_switch_fs (main_win, fs);
 	gnome_cmd_file_selector_set_connection (fs, con, NULL);
 }
 
@@ -1209,22 +1234,26 @@ on_list_key_pressed_private              (GtkCList *clist,
 										  GdkEventKey *event,
 										  GnomeCmdFileSelector *fs)
 {
-	if (!gnome_cmd_data_get_cmdline_visibility ())
-		return FALSE;
-	
-	if (state_is_blank (event->state)) {
+	if (state_is_blank (event->state) || state_is_shift (event->state)) {
 		if ((event->keyval >= GDK_A && event->keyval <= GDK_Z)
-			|| (event->keyval >= GDK_a && event->keyval <= GDK_z)) {
+			|| (event->keyval >= GDK_a && event->keyval <= GDK_z)
+			|| event->keyval == GDK_period) {
 			static gchar text[2];
-			text[0] = event->keyval;
-			text[1] = '\0';
-			gnome_cmd_cmdline_append_text (
-				gnome_cmd_main_win_get_cmdline (main_win), text);
-			gnome_cmd_cmdline_focus (
-				gnome_cmd_main_win_get_cmdline (main_win));
+
+			if (!gnome_cmd_data_get_cmdline_visibility ()) {
+				gnome_cmd_file_list_show_quicksearch (fs->list, (gchar)event->keyval);
+			}
+			else {
+				text[0] = event->keyval;
+				text[1] = '\0';
+				gnome_cmd_cmdline_append_text (
+					gnome_cmd_main_win_get_cmdline (main_win), text);
+				gnome_cmd_cmdline_focus (
+					gnome_cmd_main_win_get_cmdline (main_win));
+			}
 			return TRUE;
 		}
-	}		
+	}
 
 	return FALSE;
 }
@@ -1234,6 +1263,7 @@ static void
 on_root_btn_clicked                      (GtkButton *button,
 										  GnomeCmdFileSelector *fs)
 {
+	gnome_cmd_main_win_switch_fs (main_win, fs);
 	goto_directory (fs, "/");
 }
 
@@ -1242,77 +1272,8 @@ static void
 on_parent_btn_clicked                    (GtkButton *button,
 										  GnomeCmdFileSelector *fs)
 {
-	goto_directory (fs, "..");	
-}
-
-
-static void
-on_dir_history_popup_hide (GtkMenu *menu, GnomeCmdFileSelector *fs)
-{
-	fs->priv->dir_history_popup = NULL;
-}
-
-
-static void
-on_dir_history_item_selected (GtkMenuItem *item, const gchar *path)
-{
-	GnomeCmdFileSelector *fs = gtk_object_get_data (GTK_OBJECT (item), "fs");
-
-	g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
-	g_return_if_fail (path != NULL);
-	
-	goto_directory (fs, path);
-}
-
-
-static void
-get_dir_history_popup_pos (GtkMenu *menu, gint *x, gint *y, gboolean push_in, GnomeCmdFileSelector *fs)
-{
-	GtkWidget *wid;
-
-	wid = GTK_WIDGET (fs->list);
-
-	gdk_window_get_origin (wid->window, x, y);
-}
-
-
-static void
-popup_dir_history (GnomeCmdFileSelector *fs)
-{
-	GList *l;
-
-	if (fs->priv->dir_history_popup) return;
-	
-	fs->priv->dir_history_popup = gtk_menu_new ();
-	gtk_widget_ref (fs->priv->dir_history_popup);
-	gtk_object_set_data_full (
-		GTK_OBJECT (fs), "dir_history_popup",
-		fs->priv->dir_history_popup, (GtkDestroyNotify) gtk_widget_unref);
-	gtk_signal_connect (
-		GTK_OBJECT (fs->priv->dir_history_popup), "hide",
-		GTK_SIGNAL_FUNC (on_dir_history_popup_hide), fs);
-
-	l = fs->priv->dir_history->ents;
-	while (l) {
-		gchar *path = (gchar*)l->data;
-		GtkWidget *item = gtk_menu_item_new_with_label (path);
-		gtk_widget_ref (item);
-		gtk_object_set_data (GTK_OBJECT (item), "fs", fs);			
-		gtk_object_set_data_full (
-			GTK_OBJECT (fs->priv->dir_history_popup),
-			"menu_item", item, (GtkDestroyNotify) gtk_widget_unref);		
-		gtk_signal_connect (
-			GTK_OBJECT (item), "activate",
-			GTK_SIGNAL_FUNC (on_dir_history_item_selected), path);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (fs->priv->dir_history_popup), item);
-		l = l->next;
-	}
-
-	gnome_popup_menu_do_popup (
-		fs->priv->dir_history_popup,
-		(GtkMenuPositionFunc)get_dir_history_popup_pos,	fs,
-		NULL, NULL, NULL);
+	gnome_cmd_main_win_switch_fs (main_win, fs);
+	goto_directory (fs, "..");
 }
 
 
@@ -1383,7 +1344,6 @@ init (GnomeCmdFileSelector *fs)
 	fs->priv->dir_history = NULL;
 	fs->priv->active = FALSE;
 	fs->priv->sym_file = NULL;
-	fs->priv->dir_history_popup = NULL;
 	fs->priv->con = NULL;
 	fs->priv->con_open_dialog = NULL;
 	fs->priv->con_open_dialog_label = NULL;
@@ -2148,7 +2108,8 @@ gnome_cmd_file_selector_keypressed (GnomeCmdFileSelector *fs,
 		switch (event->keyval)
 		{
 			case GDK_Down:
-				popup_dir_history (fs);
+				gnome_cmd_dir_indicator_show_history (
+					GNOME_CMD_DIR_INDICATOR (fs->dir_indicator));
 				return TRUE;
 
 			case GDK_Left:
@@ -2203,6 +2164,10 @@ gnome_cmd_file_selector_keypressed (GnomeCmdFileSelector *fs,
 			case GDK_KP_Enter:
 				add_file_to_cmdline (fs, FALSE);
 				return TRUE;
+
+			case GDK_period:
+				set_same_directory (fs);
+				return TRUE;
 		}
 	}
 	else if (state_is_blank (event->state)) {
