@@ -59,6 +59,7 @@ struct _GnomeCmdMainWinPrivate
 	GtkWidget *vbox;
 	GtkWidget *paned;
 	GtkWidget *file_selector[2];
+	GtkWidget *focused_widget;
 	GtkAccelGroup *accel_group;
 
 	GtkWidget *back_btn;
@@ -353,10 +354,10 @@ on_help_clicked                        (GtkButton       *button,
 
 
 static void
-on_refresh_clicked                        (GtkButton       *button,
-									   GnomeCmdMainWin *mw)
+on_rename_clicked                      (GtkButton       *button,
+										GnomeCmdMainWin *mw)
 {
-	view_refresh (NULL, mw);
+	file_rename (NULL, mw);
 }
 
 
@@ -420,7 +421,7 @@ static void
 on_quit_clicked                        (GtkButton       *button,
                                         GnomeCmdMainWin *mw)
 {
-	gtk_widget_destroy (GTK_WIDGET (mw));
+	file_exit (NULL, mw);
 }
 
 
@@ -586,7 +587,7 @@ on_size_allocate                  (GtkWidget *widget,
 								   GtkAllocation *allocation,
 								   gpointer user_data)
 {
-	gnome_cmd_data_set_position (allocation->width, allocation->height);
+	gnome_cmd_data_set_main_win_size (allocation->width, allocation->height);
 }
 
 
@@ -681,6 +682,27 @@ on_fs_dir_change                  (GnomeCmdFileSelector *fs,
 }
 
 
+static void
+restore_size_and_pos (GnomeCmdMainWin *mw)
+{
+	gint w, h, x, y;
+	
+	gnome_cmd_data_get_main_win_size (&w, &h);
+	gtk_widget_set_usize (GTK_WIDGET (main_win), w, h);
+
+	gnome_cmd_data_get_main_win_pos (&x, &y);
+	if (x >= 0 && y >= 0)
+		gtk_window_move (GTK_WINDOW (mw), x, y);
+}
+
+
+static void
+on_delete_event (GnomeCmdMainWin *mw, GdkEvent *event, gpointer user_data)
+{
+	file_exit (NULL, mw);
+}
+
+
 /*******************************
  * Gtk class implementation
  *******************************/
@@ -705,7 +727,7 @@ destroy (GtkObject *object)
 	if (con_home == gnome_cmd_dir_get_connection (dir))
 		gnome_cmd_data_set_start_dir (
 			1, gnome_cmd_file_get_path (GNOME_CMD_FILE (dir)));
-		
+
 	gtk_main_quit ();
 }
 
@@ -746,8 +768,6 @@ class_init (GnomeCmdMainWinClass *class)
 static void
 init (GnomeCmdMainWin *mw)
 {
-	gint main_win_width, main_win_height;
-	
 	/* It is very important that this global variable gets assigned here so that
 	 * child widgets to this window can use that variable when initializing
 	 */
@@ -758,17 +778,18 @@ init (GnomeCmdMainWin *mw)
 	mw->priv->accel_group = gtk_accel_group_new ();
 	mw->priv->toolbar = NULL;
 	mw->priv->toolbar_sep = NULL;
+	mw->priv->focused_widget = NULL;
 	mw->priv->cmdline = NULL;
 	mw->priv->cmdline_sep = NULL;
 	mw->priv->buttonbar = NULL;
 	mw->priv->buttonbar_sep = NULL;
-
-	gnome_cmd_data_get_position (&main_win_width, &main_win_height);
+	mw->priv->file_selector[0] = NULL;
+	mw->priv->file_selector[1] = NULL;
 
 	gnome_app_construct (GNOME_APP (main_win), "gnome-commander", "GNOME Commander");
 	gtk_object_set_data (GTK_OBJECT (main_win), "main_win", main_win);
+	restore_size_and_pos (mw);
 	gtk_window_set_policy (GTK_WINDOW (main_win), TRUE, TRUE, FALSE);
-	gtk_widget_set_usize (GTK_WIDGET (main_win), main_win_width, main_win_height);
 
 	mw->priv->vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_ref (mw->priv->vbox);
@@ -864,6 +885,9 @@ init (GnomeCmdMainWin *mw)
 	gtk_signal_connect (
 		GTK_OBJECT (mw), "size-allocate",
 		GTK_SIGNAL_FUNC (on_size_allocate), mw);
+	gtk_signal_connect (
+		GTK_OBJECT (mw), "delete-event",
+		GTK_SIGNAL_FUNC (on_delete_event), mw);
 	gtk_signal_connect (
 		GTK_OBJECT (mw->priv->paned),
 		"button_press_event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);	
@@ -879,6 +903,7 @@ init (GnomeCmdMainWin *mw)
 		gnome_cmd_con_list_get_home (gnome_cmd_data_get_con_list ()), NULL);
 
 	gtk_window_add_accel_group (GTK_WINDOW (main_win), mw->priv->accel_group);
+	gnome_cmd_main_win_focus_file_lists (main_win);
 }
 
 
@@ -922,6 +947,11 @@ gnome_cmd_main_win_new              ()
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_active_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+
+	if (!mw->priv->file_selector[mw->priv->current_fs])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[mw->priv->current_fs]);
 }
 
@@ -929,6 +959,11 @@ gnome_cmd_main_win_get_active_fs         (GnomeCmdMainWin *mw)
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_inactive_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+
+	if (!mw->priv->file_selector[!mw->priv->current_fs])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[!mw->priv->current_fs]);
 }
 
@@ -936,6 +971,11 @@ gnome_cmd_main_win_get_inactive_fs         (GnomeCmdMainWin *mw)
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_left_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+	
+	if (!mw->priv->file_selector[LEFT])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]);
 }
 
@@ -943,6 +983,11 @@ gnome_cmd_main_win_get_left_fs         (GnomeCmdMainWin *mw)
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_right_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+	
+	if (!mw->priv->file_selector[RIGHT])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]);
 }
 
@@ -966,8 +1011,10 @@ gnome_cmd_main_win_update_style          (GnomeCmdMainWin *mw)
 void
 gnome_cmd_main_win_focus_cmdline         (GnomeCmdMainWin *mw)
 {
-	if (gnome_cmd_data_get_cmdline_visibility ())
+	if (gnome_cmd_data_get_cmdline_visibility ()) {
 		gnome_cmd_cmdline_focus (GNOME_CMD_CMDLINE (mw->priv->cmdline));
+		mw->priv->focused_widget = mw->priv->cmdline;
+	}
 }
 
 
@@ -978,6 +1025,16 @@ gnome_cmd_main_win_focus_file_lists      (GnomeCmdMainWin *mw)
 		gnome_cmd_main_win_get_active_fs (mw), TRUE);
 	gnome_cmd_file_selector_set_active (
 		gnome_cmd_main_win_get_inactive_fs (mw), FALSE);
+	
+	mw->priv->focused_widget = mw->priv->file_selector[mw->priv->current_fs];
+}
+
+
+void
+gnome_cmd_main_win_refocus               (GnomeCmdMainWin *mw)
+{
+	if (mw->priv->focused_widget)
+		gtk_widget_grab_focus (mw->priv->focused_widget);
 }
 
 
@@ -1137,7 +1194,7 @@ gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
 				on_help_clicked (NULL, mw);
 				return TRUE;
 			case GDK_F2:
-				on_refresh_clicked (NULL, mw);
+				on_rename_clicked (NULL, mw);
 				return TRUE;
 			case GDK_F3:
 				on_view_clicked (NULL, mw);
